@@ -2,14 +2,6 @@
 
 '''SMTP/ESMTP client class.
 
-Author: The Dragon De Monsyne <dragondm@integral.org>
-ESMTP support, test code and doc fixes added by
-    Eric S. Raymond <esr@thyrsus.com>
-Better RFC 821 compliance (MAIL and RCPT, and CRLF in data)
-    by Carey Evans <c.evans@clear.net.nz>, for picky mail servers.
-   
-This was modified from the Python 1.5 library HTTP lib.
-
 This should follow RFC 821 (SMTP) and RFC 1869 (ESMTP).
 
 Notes:
@@ -39,6 +31,14 @@ Example:
   >>> s.quit()
 '''
 
+# Author: The Dragon De Monsyne <dragondm@integral.org>
+# ESMTP support, test code and doc fixes added by
+#     Eric S. Raymond <esr@thyrsus.com>
+# Better RFC 821 compliance (MAIL and RCPT, and CRLF in data)
+#     by Carey Evans <c.evans@clear.net.nz>, for picky mail servers.
+#    
+# This was modified from the Python 1.5 library HTTP lib.
+
 import socket
 import string
 import re
@@ -77,7 +77,7 @@ class SMTPResponseException(SMTPException):
 class SMTPSenderRefused(SMTPResponseException):
     """Sender address refused.
     In addition to the attributes set by on all SMTPResponseException
-    exceptions, this sets `sender' to the string that the SMTP refused
+    exceptions, this sets `sender' to the string that the SMTP refused.
     """
 
     def __init__(self, code, msg, sender):
@@ -87,8 +87,8 @@ class SMTPSenderRefused(SMTPResponseException):
         self.args = (code, msg, sender)
 
 class SMTPRecipientsRefused(SMTPException):
-    """All recipient  addresses refused.
-    The errors for each recipient are accessable thru the attribute
+    """All recipient addresses refused.
+    The errors for each recipient are accessible through the attribute
     'recipients', which is a dictionary of exactly the same sort as 
     SMTP.sendmail() returns.  
     """
@@ -98,15 +98,15 @@ class SMTPRecipientsRefused(SMTPException):
         self.args = ( recipients,)
 
 
-
 class SMTPDataError(SMTPResponseException):
     """The SMTP server didn't accept the data."""
 
 class SMTPConnectError(SMTPResponseException):
-    """Error during connection establishment"""
+    """Error during connection establishment."""
 
 class SMTPHeloError(SMTPResponseException):
-    """The server refused our HELO reply"""
+    """The server refused our HELO reply."""
+
 
 def quoteaddr(addr):
     """Quote a subset of the email addresses defined by RFC 821.
@@ -127,22 +127,40 @@ def quoteaddr(addr):
 def quotedata(data):
     """Quote data for email.
 
-    Double leading '.', and change Unix newline '\n', or Mac '\r' into
+    Double leading '.', and change Unix newline '\\n', or Mac '\\r' into
     Internet CRLF end-of-line.
     """
     return re.sub(r'(?m)^\.', '..',
         re.sub(r'(?:\r\n|\n|\r(?!\n))', CRLF, data))
 
+def _get_fqdn_hostname(name):
+    name = string.strip(name)
+    if len(name) == 0:
+        name = socket.gethostname()
+        try:
+            hostname, aliases, ipaddrs = socket.gethostbyaddr(name)
+        except socket.error:
+            pass
+        else:
+            aliases.insert(0, hostname)
+            for name in aliases:
+                if '.' in name:
+                    break
+            else:
+                name = hostname
+    return name
+
+
 class SMTP:
     """This class manages a connection to an SMTP or ESMTP server.
     SMTP Objects:
         SMTP objects have the following attributes:    
             helo_resp 
-                This is the message given by the server in responce to the 
+                This is the message given by the server in response to the 
                 most recent HELO command.
                 
             ehlo_resp
-                This is the message given by the server in responce to the 
+                This is the message given by the server in response to the 
                 most recent EHLO command. This is usually multiline.
 
             does_esmtp 
@@ -151,17 +169,17 @@ class SMTP:
 
             esmtp_features 
                 This is a dictionary, which, if the server supports ESMTP,
-                will  _after you do an EHLO command_, contain the names of the
-                SMTP service  extentions this server supports, and their 
+                will _after you do an EHLO command_, contain the names of the
+                SMTP service extensions this server supports, and their
                 parameters (if any).
-                Note, all extention names are mapped to lower case in the 
+
+                Note, all extension names are mapped to lower case in the 
                 dictionary. 
 
-        For method docs, see each method's docstrings. In general, there is 
-            a method of the same name to perform each SMTP command, and there 
-            is a method called 'sendmail' that will do an entire mail 
-            transaction.
-    """
+        See each method's docstrings for details.  In general, there is a
+        method of the same name to perform each SMTP command.  There is also a
+        method called 'sendmail' that will do an entire mail transaction.
+        """
     debuglevel = 0
     file = None
     helo_resp = None
@@ -173,8 +191,8 @@ class SMTP:
 
         If specified, `host' is the name of the remote host to which to
         connect.  If specified, `port' specifies the port to which to connect.
-        By default, smtplib.SMTP_PORT is used.  An SMTPConnectError is
-        raised if the specified `host' doesn't respond correctly.
+        By default, smtplib.SMTP_PORT is used.  An SMTPConnectError is raised
+        if the specified `host' doesn't respond correctly.
 
         """
         self.esmtp_features = {}
@@ -213,7 +231,11 @@ class SMTP:
         if not port: port = SMTP_PORT
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         if self.debuglevel > 0: print 'connect:', (host, port)
-        self.sock.connect(host, port)
+        try:
+            self.sock.connect((host, port))
+        except socket.error:
+            self.close()
+            raise
         (code,msg)=self.getreply()
         if self.debuglevel >0 : print "connect:", msg
         return (code,msg)
@@ -223,7 +245,9 @@ class SMTP:
         if self.debuglevel > 0: print 'send:', `str`
         if self.sock:
             try:
-                self.sock.send(str)
+                sendptr = 0
+                while sendptr < len(str):
+                    sendptr = sendptr + self.sock.send(str[sendptr:])
             except socket.error:
                 raise SMTPServerDisconnected('Server not connected')
         else:
@@ -231,7 +255,10 @@ class SMTP:
  
     def putcmd(self, cmd, args=""):
         """Send a command to the server."""
-        str = '%s %s%s' % (cmd, args, CRLF)
+        if args == "":
+            str = '%s%s' % (cmd, CRLF)
+        else:
+            str = '%s %s%s' % (cmd, args, CRLF)
         self.send(str)
     
     def getreply(self):
@@ -285,10 +312,7 @@ class SMTP:
         Hostname to send for this command defaults to the FQDN of the local
         host.
         """
-        name=string.strip(name)
-        if len(name)==0:
-            name=socket.gethostbyaddr(socket.gethostname())[0]
-        self.putcmd("helo",name)
+        self.putcmd("helo", _get_fqdn_hostname(name))
         (code,msg)=self.getreply()
         self.helo_resp=msg
         return (code,msg)
@@ -298,10 +322,7 @@ class SMTP:
         Hostname to send for this command defaults to the FQDN of the local
         host.
         """
-        name=string.strip(name)
-        if len(name)==0:
-            name=socket.gethostbyaddr(socket.gethostname())[0]
-        self.putcmd("ehlo",name)
+        self.putcmd("ehlo", _get_fqdn_hostname(name))
         (code,msg)=self.getreply()
         # According to RFC1869 some (badly written) 
         # MTA's will disconnect on an ehlo. Toss an exception if 
@@ -312,7 +333,7 @@ class SMTP:
         if code<>250:
             return (code,msg)
         self.does_esmtp=1
-        #parse the ehlo responce -ddm
+        #parse the ehlo response -ddm
         resp=string.split(self.ehlo_resp,'\n')
         del resp[0]
         for each in resp:
@@ -345,8 +366,8 @@ class SMTP:
         """SMTP 'mail' command -- begins mail xfer session."""
         optionlist = ''
         if options and self.does_esmtp:
-            optionlist = string.join(options, ' ')
-        self.putcmd("mail", "FROM:%s %s" % (quoteaddr(sender) ,optionlist))
+            optionlist = ' ' + string.join(options, ' ')
+        self.putcmd("mail", "FROM:%s%s" % (quoteaddr(sender) ,optionlist))
         return self.getreply()
 
     def rcpt(self,recip,options=[]):
@@ -413,16 +434,15 @@ class SMTP:
         fails, HELO will be tried and ESMTP options suppressed.
 
         This method will return normally if the mail is accepted for at least
-        one recipient. It returns a dictionary, with one entry for each
-        recipient that was refused.  Each entry contains a tuple of
-        the SMTP error code and the accompanying error message sent by
-        the server.
+        one recipient.  It returns a dictionary, with one entry for each
+        recipient that was refused.  Each entry contains a tuple of the SMTP
+        error code and the accompanying error message sent by the server.
 
         This method may raise the following exceptions:
 
          SMTPHeloError          The server didn't reply properly to
                                 the helo greeting. 
-         SMTPRecipientsRefused  The server rejected for ALL recipients
+         SMTPRecipientsRefused  The server rejected ALL recipients
                                 (no mail was sent).
          SMTPSenderRefused      The server didn't accept the from_addr.
          SMTPDataError          The server replied with an unexpected
@@ -447,7 +467,7 @@ class SMTP:
         
         In the above example, the message was accepted for delivery to three
         of the four addresses, and one was rejected, with the error code
-        550. If all addresses are accepted, then the method will return an
+        550.  If all addresses are accepted, then the method will return an
         empty dictionary.
 
         """
