@@ -1,6 +1,6 @@
 /* common.c --- Common routines, constants, etc.  Used by all the wrappers.
  *
- * Copyright (C) 1998 by the Free Software Foundation, Inc.
+ * Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -62,7 +62,7 @@ fatal(const char* ident, int exitcode, char* format, ...)
 {
 #ifndef HAVE_VSNPRINTF
 	/* a replacement is provided in vsnprintf.c */
-	int vsnprintf(char* s, int n, char* fmt, ...);
+	int vsnprintf(char*, int, char*, va_list);
 #endif /* !HAVE_VSNPRINTF */
 
 	char log_entry[BUFSIZE];
@@ -78,7 +78,7 @@ fatal(const char* ident, int exitcode, char* format, ...)
 	 * should definitely know about any problems.
 	 */
 	openlog(ident, LOG_CONS, LOG_MAIL);
-	syslog(LOG_ERR, "%s", log_entry);
+	syslog(LOG_ERR, "%s\n", log_entry);
 	closelog();
 #endif /* HAVE_SYSLOG */
 
@@ -86,6 +86,9 @@ fatal(const char* ident, int exitcode, char* format, ...)
 	/* If we're running as a CGI script, we also want to write the log
 	 * file out as HTML, so the admin who is probably trying to debug his
 	 * installation will have a better clue as to what's going on.
+	 *
+	 * Otherwise, print to stderr a short message, hopefully returned to
+	 * the sender by the MTA.
 	 */
 	if (running_as_cgi) {
 		printf("Content-type: text/html\n\n");
@@ -104,6 +107,8 @@ fatal(const char* ident, int exitcode, char* format, ...)
 		printf(log_entry);
 		printf("</pre>\n");
 	}
+	else
+		fprintf(stderr, "%s\n", log_entry);
 #endif /* HELPFUL */
 	exit(exitcode);
 }
@@ -113,24 +118,45 @@ fatal(const char* ident, int exitcode, char* format, ...)
 /* Is the parent process allowed to call us?
  */
 void
-check_caller(const char* ident, gid_t parentgid)
+check_caller(const char* ident, GID_T parentgid)
 {
-	gid_t mygid = getgid();
+	GID_T mygid = getgid();
 	if (parentgid != mygid) {
 		fatal(ident, GID_MISMATCH,
-		      "Failure to exec script. WANTED gid %d, GOT gid %d.  "
-		      "(Reconfigure to take %d?)",
+		      "Failure to exec script. WANTED gid %ld, GOT gid %ld.  "
+		      "(Reconfigure to take %ld?)",
 		      parentgid, mygid, mygid);
 	}
 }
 
 
+
+/* list of environment variables which are removed from the given
+ * environment.  Some may or may not be hand crafted and passed into
+ * the execv'd environment.
+ *
+ * TBD: The logic of this should be inverted.  IOW, we should audit the
+ * Mailman CGI code for those environment variables that are used, and
+ * specifically white list them, removing all other variables.  John Viega
+ * also suggests imposing a maximum size just in case Python doesn't handle
+ * them right (which it should because Python strings have no hard limits).
+ */
+static char* killenvars[] = {
+	"PYTHONPATH=",
+	"PYTHONHOME=",
+	"PATH=",
+	NULL
+};
+    
+
 
 /* Run a Python script out of the script directory
  *
  * args[0] should be the abs path to the Python script to execute
  * argv[1:] are other args for the script
  * env may or may not contain PYTHONPATH, we'll substitute our own
+ *
+ * TBD: third argument env may not be universally portable
  */
 int
 run_script(const char* script, int argc, char** argv, char** env)
@@ -142,7 +168,7 @@ run_script(const char* script, int argc, char** argv, char** env)
 	int i, j, status;
 	char** newenv;
 	char** newargv;
-    
+
 	/* We need to set the real gid to the effective gid because there are
 	 * some Linux systems which do not preserve the effective gid across
 	 * popen() calls.  This breaks mail delivery unless the ~mailman/data
@@ -173,12 +199,20 @@ run_script(const char* script, int argc, char** argv, char** env)
 	/* okay to be a little too big */
 	newenv = (char**)malloc(sizeof(char*) * (envcnt + 2));
 
-	/* filter out any existing PYTHONPATH in the environment */
-	for (i = 0, j = 0; i < envcnt; i++)
-		if (strncmp(envstr, env[i], envlen)) {
-			newenv[j] = env[i];
-			j++;
+	/* filter out any troublesome environment variables */
+	for (i = 0, j = 0; i < envcnt; i++) {
+		char** k = &killenvars[0];
+		int keep = 1;
+		while (*k) {
+			if (!strncmp(*k, env[i], strlen(*k))) {
+				keep = 0;
+				break;
+			}
+			*k++;
 		}
+		if (keep)
+			newenv[j++] = env[i];
+	}
 
 	/* Tack on our own version of PYTHONPATH, which should contain only
 	 * the path to the Mailman package modules.
